@@ -8,7 +8,7 @@
 
 /* structure to hold estimation results */
 typedef struct LME_struct {
-    DIMS dd;            /* dimension data info */
+    DIMS dm;            /* dimension data info */
     LENGTHS lengths;    /* lenghts and offsets object */
     FAMILY family;      /* family information */
     double
@@ -29,16 +29,17 @@ typedef struct LME_struct {
       *dims,        /* dimensions passed from R */
       npar,         /* total length of the mixture parameters */
       maxIter,      /* maximun number of iterations */
+      fixShape,     /* must estimate shape parameters? */
       ndraws,       /* independent draws for Monte Carlo integration */
       algorithm,    /* logical flag, EM = 0, nested EM = 1 */
       ncycles;      /* number of cycles for the nested EM algorithm */
     double
       tolerance;    /* convergence tolerance */
-} LME_struct, *LMEStruct;
+} LME_struct, *LME;
 
 /* structure to hold fitted values */
 typedef struct FITTED_struct {
-    DIMS dd;            /* dimension data info */
+    DIMS dm;            /* dimension data info */
     LENGTHS lengths;    /* lenghts and offsets object */
     double
       *ZX,          /* model matrix */
@@ -50,7 +51,7 @@ typedef struct FITTED_struct {
 
 /* structure to hold the covariance of coefficients */
 typedef struct ACOV_struct {
-    DIMS dd;            /* dimension data info */
+    DIMS dm;            /* dimension data info */
     LENGTHS lengths;    /* lenghts and offsets object */
     FAMILY family;      /* family information */
     double
@@ -64,42 +65,8 @@ typedef struct ACOV_struct {
 } ACOV_struct, *ACOV;
 
 /* estimation in lme under heavy tailed distributions (to be called by R) */
-extern void heavyLme_fit(double *, double *, double *, int *, int *, int *, double *, double *, double *, double *, double *, double *, double *, double *, double *, double *);
-extern void heavyLme_fitted(double *, int *, int *, int *, double *, double *, double *, double *);
-extern void heavyLme_acov(double *, int *, int *, int *, double *, double *, double *, double *, double *);
-
-/* initialization and pre-decomposition */
-LMEStruct heavyLme_init(double *, double *, double *, int *, int *, int *, double *, double *, double *, double *, double *, double *, double *, double *, double *);
-void heavyLme_free(LMEStruct);
-void pre_decomp(double *, double *, double *, DIMS, LENGTHS);
-
-/* routines used in the iterative procedure */
-int heavyLme_iterate(LMEStruct);
-void internal_EMcycle(LMEStruct);
-void internal_Estep(LMEStruct);
-void outer_Estep(LMEStruct);
-
-/* routines used in (internal) EM cycles */
-void append_decomp(double *, double *, int, int, int, double *, int, double *, int);
-void random_effects(double *, int, DIMS, double *, double *);
-double mahalanobis(double *, int, DIMS, double *, double *);
-void working_response(double *, double *, int, DIMS, double *, double *);
-void update_coef(LMEStruct);
-void update_scale(LMEStruct);
-void update_theta(LMEStruct);
-
-/* relative precision factor and evaluation of marginal log-likelihood */
-void relative_precision(double *, int, double *, double *);
-double heavyLme_logLik(LMEStruct);
-
-/* Fisher information matrix for the coefficients */
-ACOV heavyLme_acov_init(double *, int *, int *, int *, double *, double *, double *, double *, double *);
-void heavyLme_acov_free(ACOV);
-void heavyLme_acov_coef(ACOV);
-
-/* routines for computation of fitted values */
-FITTED heavyLme_fitted_init(double *, int *, int *, int *, double *, double *, double *, double *);
-void heavyLme_fitted_free(FITTED);
-void heavyLme_fitted_values(FITTED);
+extern void lme_fit(double *, double *, double *, int *, int *, int *, double *, double *, double *, double *, double *, double *, double *, double *, double *, double *);
+extern void lme_fitted(double *, int *, int *, int *, double *, double *, double *, double *);
+extern void lme_acov(double *, int *, int *, int *, double *, double *, double *, double *, double *);
 
 #endif /* HEAVY_LME_FIT_H */
