@@ -29,6 +29,13 @@ dot_product(double *x, int incx, double *y, int incy, int n)
     return ans;
 }
 
+void
+scale(double *x, int n, int inc, double alpha)
+{   /* x <- alpha * x */
+
+    F77_CALL(dscal)(&n, &alpha, x, &inc);
+}
+
 void
 zero_mat(double *y, int ldy, int nrow, int ncol)
 {   /* y[,] <- 0 */
@@ -41,16 +48,6 @@ zero_mat(double *y, int ldy, int nrow, int ncol)
     }
 }
 
-void
-gaxpy(double *y, double alpha, double *a, int lda, int nrow, int ncol,
-    double *x, double beta)
-{   /* y <- alpha * a %*% x + beta * y */
-    char *trans = "N";
-    int inc = 1;
-
-    F77_CALL(dgemv)(trans, &nrow, &ncol, &alpha, a, &lda, x, &inc, &beta, y, &inc);
-}
-
 void
 copy_mat(double *y, int ldy, double *x, int ldx, int nrow, int ncol)
 {   /* y <- x[,] */
@@ -73,6 +70,17 @@ add_mat(double *y, int ldy, double alpha, double *x, int ldx, int nrow, int ncol
     }
 }
 
+void
+gaxpy(double *y, double alpha, double *a, int lda, int nrow, int ncol,
+    double *x, double beta, int job)
+{   /* y <- alpha * a %*% x + beta * y or y <- alpha * t(a) %*% x + beta * y */
+    char *trans;
+    int inc = 1;
+
+    trans = (job) ? "T" : "N";
+    F77_CALL(dgemv)(trans, &nrow, &ncol, &alpha, a, &lda, x, &inc, &beta, y, &inc);
+}
+
 void
 lower_tri(double *y, int ldy, double *x, int ldx, int nrow, int ncol)
 {   /* y <- lower.tri(x) */
@@ -138,10 +146,14 @@ crossprod(double *x, int ldx, int xrows, int xcols, double *y, int ldy, int yrow
     int ycols, double *z)
 {   /* cross product of two given matrices. z <- t(x) %*% y */
     char *transx = "T", *transy = "N";
-    double one = 1.0, zero = 0.0;
+    double one = 1.0, zero = 0.0, *tmp = NULL;
 
+    /* use tmp so z can be either x or y */
+    tmp = (double *) Calloc(xcols * ycols, double);
     F77_CALL(dgemm)(transx, transy, &xcols, &ycols, &xrows, &one, x, &ldx, y,
-                    &ldy, &zero, z, &xcols);
+                    &ldy, &zero, tmp, &xcols);
+    Memcpy(z, tmp, xcols * ycols);
+    Free(tmp);
 }
 
 void
@@ -178,24 +190,34 @@ logAbsDet(double *a, int lda, int n)
 /* routines for matrix decompositions */
 
 void
-chol_decomp(int job, double *a, int lda, int n)
+chol_decomp(double *a, int lda, int p, int job, int *info)
 {   /* cholesky factorization of a real symmetric positive definite matrix a.
      * the factorization has the form: a <- l  %*% t(l), if job = 0, or
      * a <- t(u) %*% u, if job = 1, where u is an upper triangular matrix and
      * l is lower triangular. */
-    int info = 0;
     char *uplo;
-
+    
     uplo = (job) ? "U" : "L";
-    F77_CALL(dpotrf)(uplo, &n, a, &lda, &info);
-    if (info)
-        error("DPOTRF in cholesky decomposition gave code %d", info);
+    F77_CALL(dpotrf)(uplo, &p, a, &lda, info);
+}
+
+void
+svd_decomp(double *mat, int ldmat, int nrow, int ncol, double *u, int ldu, double *d,
+           double *v, int ldv, int job, int *info)
+{   /* return the SVD decomposition of mat. */
+    double *work, *upper;
+
+    work  = (double *) Calloc(nrow, double);
+    upper = (double *) Calloc(ncol, double);    
+    F77_CALL(dsvdc)(mat, &ldmat, &nrow, &ncol, d, upper, u, &ldu, v, &ldv,
+                    work, &job, info);
+    Free(work); Free(upper);
 }
 
 QRStruct
 QR_decomp(double *mat, int ldmat, int nrow, int ncol, double *qraux)
 {   /* return the QR decomposition of mat */
-    int info = 0;
+    int *dummy = NULL, job = 0;
     double *work;
     QRStruct value;
 
@@ -206,10 +228,8 @@ QR_decomp(double *mat, int ldmat, int nrow, int ncol, double *qraux)
     value->nrow  = nrow;
     value->ncol  = ncol;
     value->qraux = qraux;
-    F77_CALL(dgeqrf)(&nrow, &ncol, mat, &ldmat, qraux, work, &ncol, &info);
+    F77_CALL(dqrdc)(mat, &ldmat, &nrow, &ncol, qraux, dummy, work, &job);
     Free(work);
-    if (info)
-        error("DGEQRF in QR decomposition gave code %d", info);
     return value;
 }
 
@@ -222,35 +242,70 @@ QR_free(QRStruct this)
 /* orthogonal-triangular operations */
 
 void
-QR_qty(QRStruct this, double *ymat, int ldy, int yrow, int ycol)
-{   /* ymat <- qr.qty(this, ymat) */
-    int info = 0, nrflc, lwork;
-    char *side = "L", *trans = "T";
-    double *work;
+QR_qty(QRStruct this, double *ymat, int yrow, int ycol, double *qty)
+{   /* qty <- qr.qty(this, ymat) */
+    int j, job = 1000, info = 0;
+    double *dummy = NULL;
 
-    nrflc = MIN(yrow, this->ncol);
-    lwork = MAX(ycol, this->ncol);
-    work  = (double *) Calloc(lwork, double);
-    F77_CALL(dormqr)(side, trans, &yrow, &ycol, &nrflc, this->mat, &(this->ldmat),
-                     this->qraux, ymat, &ldy, work, &lwork, &info);
-    Free(work);
-    if (info)
-        error("DORMQR in QR_qty gave code %d", info);
+    for (j = 0; j < ycol; j++) {
+        F77_CALL(dqrsl)(this->mat, &(this->ldmat), &(this->nrow), &(this->ncol),
+                        this->qraux, ymat + j * yrow, dummy, qty + j * yrow,
+                        dummy, dummy, dummy, &job, &info);
+    }
 }
 
 void
-QR_qy(QRStruct this, double *ymat, int yrow, int ycol)
-{   /* ymat <- qr.qy(this, ymat) */
-    int info = 0;
-    char *side = "L", *notrans = "N";
-    double *work;
+QR_qy(QRStruct this, double *ymat, int yrow, int ycol, double *qy)
+{   /* qy <- qr.qy(this, ymat) */
+    int j, job = 10000, info = 0;
+    double *dummy = NULL;
 
-    work = (double *) Calloc(ycol, double);
-    F77_CALL(dormqr)(side, notrans, &yrow, &ycol, &yrow, this->mat, &(this->ldmat),
-                     this->qraux, ymat, &yrow, work, &ycol, &info);
-    Free(work);
-    if (info)
-        error("DORMQR in QR_qy gave code %d", info);
+    for (j = 0; j < ycol; j++) {
+        F77_CALL(dqrsl)(this->mat, &(this->ldmat), &(this->nrow), &(this->ncol),
+                        this->qraux, ymat + j * yrow, qy + j * yrow, dummy,
+                        dummy, dummy, dummy, &job, &info);
+    }
+}
+
+void
+QR_coef(QRStruct this, double *ymat, int yrow, int ycol, double *coef)
+{   /* coef <- qr.coef(this, ymat) */
+    int j, job = 100, info = 0;
+    double *dummy = NULL;
+
+    for (j = 0; j < ycol; j++) {
+        F77_CALL(dqrsl)(this->mat, &(this->ldmat), &(this->nrow), &(this->ncol),
+                        this->qraux, ymat + j * yrow, dummy, ymat + j * yrow,
+                        coef + j * this->ncol, dummy, dummy, &job, &info);
+        if (info)
+            error("DQRSL in QR_coef gave code %d", info);
+    }
+}
+
+void
+QR_resid(QRStruct this, double *ymat, int yrow, int ycol, double *resid)
+{   /* resid <- qr.resid(this, ymat) */
+    int j, job = 10, info = 0;
+    double *dummy = NULL;
+    
+    for (j = 0; j < ycol; j++) {
+        F77_CALL(dqrsl)(this->mat, &(this->ldmat), &(this->nrow), &(this->ncol),
+                        this->qraux, ymat + j * yrow, dummy, ymat + j * yrow,
+                        dummy, resid + j * yrow, dummy, &job, &info);
+    }
+}
+
+void
+QR_fitted(QRStruct this, double *ymat, int yrow, int ycol, double *fitted)
+{   /* fitted <- qr.fitted(this, ymat) */
+    int j, job = 1, info = 0;
+    double *dummy = NULL;
+
+    for (j = 0; j < ycol; j++) {
+        F77_CALL(dqrsl)(this->mat, &(this->ldmat), &(this->nrow), &(this->ncol),
+                        this->qraux, ymat + j * yrow, dummy, ymat + j * yrow,
+                        dummy, dummy, fitted + j * yrow, &job, &info);
+    }
 }
 
 void
@@ -267,70 +322,59 @@ QR_store_R(QRStruct this, double *Dest, int ldDest)
 /* matrix inversion and solver */
 
 void
-invert_mat(double *a, int lda, int n)
+invert_mat(double *a, int lda, int n, int *info)
 {   /* performs matrix inversion */
-    int info = 0, j, lwork = 2 * n;
+    int j, lwork = 2 * n;
     char *trans = "N";
     double *b, *work;
 
-    b = (double *) Calloc(n * n, double);
+    b    = (double *) Calloc(n * n, double);
     work = (double *) Calloc(lwork, double);
     for (j = 0; j < n; j++)
-        b[j * (n + 1)] = 1.0;
-    F77_CALL(dgels)(trans, &n, &n, &n, a, &lda, b, &n, work, &lwork, &info);
+        b[j * (n + 1)] = 1.;
+    F77_CALL(dgels)(trans, &n, &n, &n, a, &lda, b, &n, work, &lwork, info);
     Memcpy(a, b, n * n);
     Free(b); Free(work);
-    if (info)
-        error("DGELS in computation of matrix inverse gave code %d", info);
 }
 
 void
-invert_triangular(int job, double *a, int lda, int n)
+invert_triangular(double *a, int lda, int n, int job, int *info)
 {   /* computes the inverse of an upper (job = 1) or lower (job = 0) triangular
      * matrix in place */
-    int info = 0;
     char *diag = "N", *uplo;
 
     uplo = (job) ? "U" : "L";
-    F77_CALL(dtrtri)(uplo, diag, &n, a, &lda, &info);
-    if (info)
-        error("DTRTRI in computation of matrix inverse gave code %d", info);
+    F77_CALL(dtrtri)(uplo, diag, &n, a, &lda, info);
 }
 
 void
-backsolve(int job, double *r, int ldr, int n, double *x, int ldx, int nrhs)
-{   /* backsolve solve triangular systems of the form r %*% x = b, or
-     * t(r) %*% x = b, where r is a triangular and x is a matrix containing
-     * the right-hand sides to equations. job specifies what kind of system
-     * is to be solved: job = 00, solve r %*% x = b, r lower triangular,
-     * job = 01, solve r %*% x = b, r upper triangular, job = 10, solve
-     * t(r) %*% x = b, r lower triangular, job = 11, solve t(r) %*% x = b,
+backsolve(double *r, int ldr, int n, double *b, int ldb, int nrhs, int job, int *info)
+{   /* backsolve solve triangular systems of the form r %*% x = b, or t(r) %*% x = b,
+     * where r is a triangular and b is a matrix containing the right-hand sides to
+     * equations. job specifies what kind of system is to be solved: job = 00, solve
+     * r %*% x = b, r lower triangular, job = 01, solve r %*% x = b, r upper triangular,
+     * job = 10, solve t(r) %*% x = b, r lower triangular, job = 11, solve t(r) %*% x = b,
      * r upper triangular. */
-    int info = 0;
     char *diag = "N", *uplo, *trans;
 
     trans = ((job) / 10) ? "T" : "N";
     uplo  = ((job) % 10) ? "U" : "L";
-    F77_CALL(dtrtrs)(uplo, trans, diag, &n, &nrhs, r, &ldr, x, &ldx, &info);
-    if (info)
-        error("DTRTRS in backsolve gave code %d", info);
+    F77_CALL(dtrtrs)(uplo, trans, diag, &n, &nrhs, r, &ldr, b, &ldb, info);
 }
 
 /* linear least-squares fit */
 
 void
-lsfit(double *x, int ldx, int nrow, int ncol, double *y, int ldy, int nrhs, double *coef)
+lsfit(double *x, int ldx, int nrow, int ncol, double *y, int ldy, int nrhs,
+    double *coef, int *info)
 {   /* solve (overdeterminated) least squares problems */
     char *notrans = "N";
-    int info = 0, lwork;
+    int lwork;
     double *work;
 
     lwork = ncol + MAX(ncol, nrhs);
     work  = (double *) Calloc(lwork, double);
-    F77_CALL(dgels)(notrans, &nrow, &ncol, &nrhs, x, &ldx, y, &ldy, work, &lwork, &info);
-    if (info)
-        error("DGELS in lsfit gave code %d", info);
+    F77_CALL(dgels)(notrans, &nrow, &ncol, &nrhs, x, &ldx, y, &ldy, work, &lwork, info);
     copy_mat(coef, ncol, y, ldy, ncol, nrhs);
     Free(work);
 }
-
